import fs from "fs-extra";
import { join } from "path";
import walkSync from "walk-sync";
import {
  JSON_PATTERN,
  DEFAULT_OUT_DIR,
  PATH_ROOT_PATTERN,
} from "./constants.js";
import { diff } from "../lib/diff.js";

export const isDirectory = (path) => {
  try {
    const stat = fs.lstatSync(path);
    return stat.isDirectory();
  } catch (error) {
    return false;
  }
};

const getDiffJson = (oldStruct, newStruct) => {
  const result = diff(oldStruct, newStruct);
  // 合并新增和修改的项
  let target = {};

  result.added.forEach((item) => {
    const [keyPath, value] = item;
    set(target, keyPath, value);
  });

  result.edited.forEach((item) => {
    const [keyPath, _oldValue, newValue] = item;
    set(target, keyPath, newValue);
  });

  return target;
};

const getKeyPath = (path) => path.split("/");

const set = (object, path, value) => {
  if (typeof object !== "object") return object;
  getKeyPath(path).reduce((o, k, i, _) => {
    if (i === _.length - 1) {
      // 若遍历结束直接赋值
      o[k] = value;
      return null;
    } else if (k in o) {
      // 若存在对应路径，则返回找到的对象，进行下一次遍历
      return o[k];
    } else {
      o[k] = {};
      return o[k];
    }
  }, object);
  return object;
};

/**
 * @description 比对两个相同结构的目录，如果传入 output
 */
export const diffDir = (oldDirPath, newDirPath, option) => {
  const basePath = process.cwd();
  const isRootPath = PATH_ROOT_PATTERN.test(oldDirPath);
  const jsonFileStructs = walkSync(oldDirPath)
    .filter((filePath) => JSON_PATTERN.test(filePath))
    .map((filePath) => {
      const pathStruct = {
        basePath: "",
        filePath: "",
        oldDirPath: "",
        newDirPath: "",
      };
      pathStruct["basePath"] = basePath;
      pathStruct["filePath"] = filePath;
      pathStruct["oldDirPath"] = oldDirPath;
      pathStruct["newDirPath"] = newDirPath;

      return pathStruct;
    });

  for (const {
    basePath,
    filePath,
    oldDirPath,
    newDirPath,
  } of jsonFileStructs) {
    const oldJsonStruct = fs.readJSONSync(
      isRootPath
        ? join(oldDirPath, filePath)
        : join(basePath, oldDirPath, filePath),
      {
        throws: false,
      }
    );
    const newJsonStruct = fs.readJSONSync(
      isRootPath
        ? join(newDirPath, filePath)
        : join(basePath, newDirPath, filePath),
      {
        throws: false,
      }
    );

    if (!oldJsonStruct || !newJsonStruct) {
      continue;
    }

    const result = getDiffJson(oldJsonStruct, newJsonStruct);
    const output = option.output
      ? PATH_ROOT_PATTERN.test(option.output)
        ? join(option.output, filePath)
        : join(basePath, option.output, filePath)
      : join(basePath, DEFAULT_OUT_DIR, filePath);

    // output
    fs.outputJSONSync(output, result, { spaces: 2 });
  }
};
